@@ -9,23 +9,23 @@ from typing import Any, Dict, Iterable, Iterator, List, Optional, Tuple
 
 import yaml
 
-from dql import utils
 from dql.client import Client
 from dql.data_storage import AbstractDataStorage
 from dql.listing import Listing
 from dql.node import NodeWithPath
-from dql.storage import Storage
-from dql.utils import DQL_PATH, dql_paths_join, time_to_str
+from dql.storage import Status, Storage
+from dql.utils import DQLDir, dql_paths_join, time_to_str
 
 logger = logging.getLogger("dql")
 
-DB_FILE_NAME = "db"
 DEFAULT_DATASET_DIR = "dataset"
 DATASET_FILE_SUFFIX = ".dql"
 
 TTL_INT = 4 * 60 * 60
 
-db_file = os.path.join(DQL_PATH, DB_FILE_NAME)
+
+class PendingIndexingError(Exception):
+    pass
 
 
 class DataSource:
@@ -50,28 +50,36 @@ class NodeGroup:
 
 
 def check_output_dataset_file(
-    output: str, force: bool = False, dataset_filename: Optional[str] = None
+    output: str,
+    force: bool = False,
+    dataset_filename: Optional[str] = None,
+    skip_check_output: bool = False,
+    skip_check_dql: bool = False,
 ) -> str:
     """
     Checks the output directory and dataset filename for existence or if they
     should be force-overwritten.
     """
-    if os.path.exists(output):
-        if force:
-            shutil.rmtree(output)
-        else:
-            raise RuntimeError(f"Output directory already exists: {output}")
+    if not skip_check_output:
+        if os.path.exists(output):
+            if force:
+                shutil.rmtree(output)
+            else:
+                raise RuntimeError(
+                    f"Output directory already exists: {output}"
+                )
 
     dataset_file = (
         dataset_filename if dataset_filename else output + DATASET_FILE_SUFFIX
     )
-    if os.path.exists(dataset_file):
-        if force:
-            os.remove(dataset_file)
-        else:
-            raise RuntimeError(
-                f"Output dataset file already exists: {dataset_file}"
-            )
+    if not skip_check_dql:
+        if os.path.exists(dataset_file):
+            if force:
+                os.remove(dataset_file)
+            else:
+                raise RuntimeError(
+                    f"Output dataset file already exists: {dataset_file}"
+                )
     return dataset_file
 
 
@@ -108,17 +116,18 @@ class Catalog:
     def __init__(
         self,
         data_storage,
-        cache_dir=utils.DQL_CACHE,
-        tmp_dir=utils.DQL_CACHE_TMP,
+        cache_dir=None,
+        tmp_dir=None,
     ):
         from dvc_data.hashfile.db.local import LocalHashFileDB
         from dvc_objects.fs.local import LocalFileSystem
 
+        dql_dir = DQLDir(cache=cache_dir, tmp=tmp_dir)
         self.data_storage = data_storage
         self.cache = LocalHashFileDB(
             LocalFileSystem(),
-            cache_dir,
-            tmp_dir=tmp_dir,
+            dql_dir.cache,
+            tmp_dir=dql_dir.tmp,
         )
 
     @staticmethod
@@ -137,26 +146,58 @@ class Catalog:
         storage = client.as_record()
         source_data_storage = data_storage.clone(storage=storage)
 
+        # TODO make storage status update atomic
         cached_storage = source_data_storage.get_storage(
             storage.id, time_to_str(timestamp)
         )
+        if cached_storage:
+            if cached_storage.status == Status.CREATED:
+                must_index = True
+            elif cached_storage.status == Status.FAILED:
+                must_index = True
+            elif cached_storage.status == Status.PENDING:
+                # TODO treat as a failed job if the last node in this bucket
+                # was updated a long time ago
+                raise PendingIndexingError(
+                    "Pending indexing operation: "
+                    f"(name={cached_storage.name}, id={cached_storage.id})"
+                )
+            elif cached_storage.status == Status.COMPLETE:
+                must_index = force_update
+            else:
+                raise ValueError(
+                    f"Invalid storage status: {cached_storage.status}"
+                )
+            if not must_index:
+                logger.debug(  # type: ignore[unreachable]
+                    f"Using cached listing {cached_storage.uri}."
+                    + f" Valid till: {cached_storage.expires_to_local}"
+                )
+                # Listing has to have correct version of data storage
+                # initialized with correct Storage
+                return (
+                    Listing(cached_storage, source_data_storage, client),
+                    path,
+                )
 
-        if cached_storage and not force_update:  # type: ignore[unreachable]
-            logger.debug(  # type: ignore[unreachable]
-                f"Using cached listing {cached_storage.uri}."
-                + f" Valid till: {cached_storage.expires_to_local}"
-            )
-            # Listing has to have correct version of data storage initialized
-            # with correct Storage
-            return Listing(cached_storage, source_data_storage, client), path
-
-        source_data_storage.init_db()
         lst = Listing(storage, source_data_storage, client)
         if to_fetch:
-            source_data_storage.register_storage_for_indexing(storage)
-            lst.fetch()
-            source_data_storage.mark_storage_indexed(storage, ttl)
-
+            storage = storage._replace(status=Status.PENDING)
+            try:
+                source_data_storage.register_storage_for_indexing(storage)
+                source_data_storage.init_db()
+                lst.fetch()
+                source_data_storage.mark_storage_indexed(
+                    storage.id, Status.COMPLETE, ttl
+                )
+            except:  # noqa: E722,B001
+                source_data_storage.mark_storage_indexed(
+                    storage.id, Status.FAILED, ttl
+                )
+                raise
+            lst.storage = storage
+        else:
+            source_data_storage.init_db()
         return lst, path
 
     @staticmethod
@@ -298,10 +339,11 @@ class Catalog:
         dql_file: Optional[str] = None,
         dql_only: bool = False,
         no_glob: bool = False,
+        no_dql_file: bool = False,
         ttl: int = TTL_INT,
         *,
         client_config=None,
-    ) -> None:
+    ) -> List[Dict[str, Any]]:
         node_groups = Catalog.enlist_sources_grouped(
             self.data_storage,
             sources,
@@ -311,7 +353,9 @@ class Catalog:
             client_config=client_config,
         )
 
-        dataset_file = check_output_dataset_file(output, force, dql_file)
+        dataset_file = check_output_dataset_file(
+            output, force, dql_file, dql_only, no_dql_file
+        )
 
         metafile_data = []
 
@@ -381,15 +425,17 @@ class Catalog:
             if metafile_group["files"]:
                 metafile_data.append(metafile_group)
 
-        if not metafile_data:
-            # Don't write the metafile if nothing was copied
-            return
+        if not metafile_data or no_dql_file:
+            # Don't write the metafile if nothing was copied (or skipped)
+            return metafile_data
 
         print(f"Creating '{dataset_file}'")
 
         with open(dataset_file, "w", encoding="utf-8") as fd:
             yaml.dump(metafile_data, fd, sort_keys=False)
 
+        return metafile_data
+
     def du(
         self,
         sources,
@@ -458,9 +504,7 @@ class Catalog:
             ):
                 client, _ = Client.parse_url(bucket.uri)
                 storage = client.as_record()
-                print(
-                    f"Registering storage,name:{storage.name},id: {storage.id}"
-                )
+                print(f"Registering storage {storage.name}, {storage.id}")
                 self.data_storage.upsert_storage(storage)
 
         return Catalog.enlist_sources(
