@@ -46,7 +46,7 @@ class ClientS3(FSSpecClient):
             _, subprefix, _ = self.fs.split_path(info["name"])
             if info["type"] == "directory":
                 name = full_path.split(delimiter)[-1]
-                new_dir_id = listing.insert_dir(
+                new_dir_id = await listing.insert_dir(
                     dir_id, name, datetime.max, subprefix
                 )
                 subdirs.add((new_dir_id, subprefix))
@@ -55,11 +55,11 @@ class ClientS3(FSSpecClient):
                     self._df_from_info(info, dir_id, delimiter, subprefix)
                 )
                 if len(files) >= FETCH_CHUNKSIZE:
-                    listing.data_storage.insert(files, is_dir=False)
+                    await listing.data_storage.insert(files, is_dir=False)
                     pbar.update(len(files))
                     files = []
         if files:
-            listing.data_storage.insert(files, is_dir=False)
+            await listing.data_storage.insert(files, is_dir=False)
             pbar.update(len(files))
         pbar.update(len(subdirs))
         return subdirs
