@@ -250,13 +250,13 @@ class DefaultDataStorage(AbstractDataStorage):
                     [dir_id, *values],
                 )
 
-    def insert(self, values: List[Dict[str, Any]], **kwargs) -> None:
+    async def insert(self, values: List[Dict[str, Any]], **kwargs):
         if kwargs.get("is_dir", False):
             self._insert_directories(values)
         else:
             self._insert_files(values)
 
-    def get_dir(self, parent_id: str, name: str) -> Optional[str]:
+    async def get_dir(self, parent_id: str, name: str):
         res = self.db.execute(
             f"""
             SELECT dir_id
@@ -270,8 +270,8 @@ class DefaultDataStorage(AbstractDataStorage):
 
         return None
 
-    def insert_root(self, storage: Storage) -> int:
-        self.insert([{"dir_id": 0}], is_dir=True, is_root=True)
+    async def insert_root(self, storage: Storage):
+        await self.insert([{"dir_id": 0}], is_dir=True, is_root=True)
         # root in DuckDB has always zero id
         return 0
 
