@@ -1,81 +1,10 @@
-import os
 import shutil
-import stat
-from pathlib import PosixPath
-from typing import Dict, NamedTuple
 
 import pytest
 import yaml
-from upath.implementations.cloud import CloudPath
 
-from dql.catalog import Catalog, parse_dql_file
-
-
-class CloudTestCatalog(NamedTuple):
-    src: CloudPath
-    working_dir: PosixPath
-    catalog: Catalog
-    client_config: Dict[str, str]
-
-
-class CloudTestServer(NamedTuple):
-    src: CloudPath
-    client_config: Dict[str, str]
-
-
-# Test servers can be cached as they are not modified in these tests
-cloud_server_cache: Dict[str, CloudTestServer] = {}
-
-
-# From: https://docs.python.org/3/library/shutil.html#rmtree-example
-def remove_readonly(func, path, _):
-    "Clear the readonly bit and reattempt the removal"
-    os.chmod(path, stat.S_IWRITE)
-    func(path)
-
-
-@pytest.fixture
-def cloud_test_catalog(request, tmp_path, tmp_upath_factory, data_storage):
-    cloud_type = request.param
-    if cloud_type in cloud_server_cache:
-        src_path = cloud_server_cache[cloud_type].src
-        client_config = cloud_server_cache[cloud_type].client_config
-    else:
-        src_path = tmp_upath_factory.mktemp(cloud_type)
-        fs = src_path._accessor._fs  # pylint:disable=protected-access
-        if cloud_type == "s3":
-            endpoint_url = fs.client_kwargs["endpoint_url"]
-            client_config = {"aws_endpoint_url": endpoint_url}
-        elif cloud_type == "gcs":
-            endpoint_url = fs._endpoint  # pylint:disable=protected-access
-            client_config = {"endpoint_url": endpoint_url}
-        (src_path / "description").write_text("Cats and Dogs")
-        (src_path / "cats" / "cat1").write_text("meow")
-        (src_path / "cats" / "cat2").write_text("mrow")
-        (src_path / "dogs" / "dog1").write_text("woof")
-        (src_path / "dogs" / "dog2").write_text("arf")
-        (src_path / "dogs" / "dog3").write_text("bark")
-        (src_path / "dogs" / "others" / "dog4").write_text("ruff")
-        cloud_server_cache[cloud_type] = CloudTestServer(
-            src=src_path,
-            client_config=client_config,
-        )
-
-    cache_dir = tmp_path / ".dql" / "cache"
-    cache_dir.mkdir(parents=True)
-    tmpfile_dir = tmp_path / ".dql" / "tmp"
-    tmpfile_dir.mkdir()
-
-    catalog = Catalog(
-        data_storage, cache_dir=str(cache_dir), tmp_dir=str(tmpfile_dir)
-    )
-
-    return CloudTestCatalog(
-        src=src_path,
-        working_dir=tmp_path,
-        catalog=catalog,
-        client_config=client_config,
-    )
+from dql.catalog import parse_dql_file
+from dql.utils import remove_readonly
 
 
 @pytest.mark.parametrize("cloud_test_catalog", ["s3", "gcs"], indirect=True)
@@ -496,6 +425,23 @@ def test_cp_dql_file_options(cloud_test_catalog):
             no_glob=True,
         )
 
+    # Should succeed, as the DQL file exists check will be skipped
+    dql_only_data = catalog.cp(
+        [src_path],
+        str(dest),
+        client_config=cloud_test_catalog.client_config,
+        recursive=False,
+        dql_only=True,
+        dql_file=str(dql_file),
+        no_dql_file=True,
+    )
+
+    # Check the returned DQL data contents
+    assert len(dql_only_data) == len(dql_contents)
+    dql_only_source = dql_only_data[0]
+    assert dql_only_source["data-source"]["uri"] == src_path.rstrip("/")
+    assert dql_only_source["files"] == data["files"]
+
 
 @pytest.mark.parametrize("cloud_test_catalog", ["s3", "gcs"], indirect=True)
 def test_cp_dql_file_sources(cloud_test_catalog):
